import java.io.File;
import org.apache.mahout.cf.taste.common.TasteException;
import org.apache.mahout.cf.taste.eval.RecommenderBuilder;
import org.apache.mahout.cf.taste.eval.RecommenderEvaluator;
import org.apache.mahout.cf.taste.impl.eval.AverageAbsoluteDifferenceRecommenderEvaluator;
import org.apache.mahout.cf.taste.impl.model.file.FileDataModel;
import org.apache.mahout.cf.taste.impl.neighborhood.NearestNUserNeighborhood;
import org.apache.mahout.cf.taste.impl.recommender.GenericUserBasedRecommender;
import org.apache.mahout.cf.taste.impl.similarity.PearsonCorrelationSimilarity;
import org.apache.mahout.cf.taste.model.DataModel;
import org.apache.mahout.cf.taste.neighborhood.UserNeighborhood;
import org.apache.mahout.cf.taste.recommender.Recommender;
import org.apache.mahout.cf.taste.similarity.UserSimilarity;
import org.apache.mahout.common.RandomUtils;

//TestRecommenationのアルゴリズムを評価するための処理。
//入力されたデータを分割して、それぞれのデータを処理した結果を比較して、
//アルゴリズムの精度を確認する。
public class Evaluator {
	public static void main(String[] args) throws Exception{
		//サンプル用の処理。
		//このコードを呼び出すと後続のevaluateメソッドの処理において
		//評価用の使われるデータがランダムではなく固定されるので
		//この評価プログラムの実行結果が毎回同じとなる。
		RandomUtils.useTestSeed();

		//評価対象となるRecommenderを生成する。
		//TestRecommendationと同じ処理を実装している。
		RecommenderBuilder recommenderBuilder = new RecommenderBuilder() {
			public Recommender buildRecommender(DataModel model) throws TasteException {
		 		UserSimilarity similarity = new PearsonCorrelationSimilarity(model);
		 		UserNeighborhood neighborhood = new NearestNUserNeighborhood(2, similarity, model);
		 		Recommender recommender = new GenericUserBasedRecommender(model, neighborhood, similarity);
		 		return recommender;
		     }
		};
		
		//購買履歴のCSVファイルを取り込みます。
		DataModel dataModel = new FileDataModel(new File("resources/inputdata.csv"));
		
		//アルゴリズムの評価に使う割合を指定します。
		//0.9を指定した場合、90%のデータを使って評価を行い、
		//残り10%のデータの値と比較を行います。
        double trainingPercentage = 0.9;
        
        //評価に使う割合を指定します。1.0は100%のデータを使います。
        double evaluationPercentage = 1.0;
		
		//評価用のクラスを生成します。
		RecommenderEvaluator evaluator = new AverageAbsoluteDifferenceRecommenderEvaluator();
		
		//評価結果を取得します。
		double score = evaluator.evaluate(recommenderBuilder, null, dataModel, trainingPercentage, evaluationPercentage); 
		
		//アルゴリズムの評価結果（予測結果の誤差）を出力します。
		System.out.println("["+score+"]");
	}
}
