import java.io.File;
import java.util.List;
import org.apache.mahout.cf.taste.impl.model.file.FileDataModel;
import org.apache.mahout.cf.taste.impl.neighborhood.NearestNUserNeighborhood;
import org.apache.mahout.cf.taste.impl.recommender.GenericUserBasedRecommender;
import org.apache.mahout.cf.taste.impl.similarity.PearsonCorrelationSimilarity;
import org.apache.mahout.cf.taste.model.DataModel;
import org.apache.mahout.cf.taste.neighborhood.UserNeighborhood;
import org.apache.mahout.cf.taste.recommender.RecommendedItem;
import org.apache.mahout.cf.taste.recommender.Recommender;
import org.apache.mahout.cf.taste.similarity.UserSimilarity;

public class TestRecommendation {
	public static void main(String[] args) throws Exception{
		//Recommendationを行うユーザーID（ユーザー①）を指定しています。
		long sampleUser = 1;
		
		//購買履歴のCSVファイルを取り込みます。
		DataModel model = new FileDataModel(new File("resources/inputdata.csv"));
		
		//ピアソンの積率相関係数を使ってユーザー間の類似度を分析するクラスを呼び出す。
		UserSimilarity similarity = new PearsonCorrelationSimilarity(model);
		
		//確認のためユーザー①と他ユーザーの相関係数を出力します。
		System.out.println("++++++++++Similarity++++++++++");
		for(long userid=1; userid<=5; userid++){
			System.out.println(sampleUser + ":" + userid + " = " +
					similarity.userSimilarity(sampleUser, userid));
		}

		//類似ユーザーのオブジェクトをインスタンス化します。
		//第一引数は推薦に利用するユーザー数をしています。
		//2を指定すれば類似度の高い上位2名を推薦に利用します。
		UserNeighborhood neighborhood = new NearestNUserNeighborhood(2, similarity, model);
		
		//確認のためユーザー①の類似ユーザーを出力します。
		System.out.println("++++++++++Neighborhood++++++++++");
		long[] users = neighborhood.getUserNeighborhood(sampleUser);
		for(int i=0; i<users.length; i++){
			System.out.println(users[i]);
		}
		
		//Recommendationを行うクラスを呼び出します。
		Recommender recommender = new GenericUserBasedRecommender(model, neighborhood, similarity);
		
		//ユーザー①に対するレコメンドを取得します。
		//第二引数には推薦の数を指定します。今回は2個の推薦結果を取得します。
		System.out.println("++++++++++Recommendation++++++++++");
		List<RecommendedItem> recommendations = recommender.recommend(sampleUser, 2);
		for (RecommendedItem recommendation : recommendations) {
			//推薦結果を出力します。
			//System.out.println(recommendation);としても同様の結果となります。
		    System.out.println("RecommendedItem[item:" + recommendation.getItemID() + ", " +
		    		"value:" + recommendation.getValue() + "]");
		}
	}
}
